/*
* file CcProvider.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcProvider
*
*  Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import java.util.List;

import javax.wvcm.ResourceList;
import javax.wvcm.WorkspaceProvider;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyRequestItem.PropertyRequest;
import javax.wvcm.ProviderFactory.Callback.Authentication;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.cc.CcMergeHandlingCallback.CheckinMergeHandling;
import com.ibm.rational.wvcm.stp.cc.CcView.TextMode;
import com.ibm.rational.wvcm.stp.ccex.CcExServer;

/**
 * A ClearCase-specific extension of the javax.wvcm.Provider interface.
 * <p>
 * The CcProvider interface contains factory methods for constructing proxies
 * for various ClearCase-specific resources. Note that there are many resource
 * types where a ClearCase-specific interface exists for a standard WVCM
 * interface.  For instance, CcFile is the ClearCase specialization of
 * javax.wvcm.ControllableResource.
 * </p>
 * <p>
 * The CcProvider interface also contains methods for listing client-side
 * and server-side ClearCase resources:
 * <bl>
 * <li>{@link #getClientViewList} lists the web views
 * available on the local machine
 * </li>
 * <li>{@link #doGetDefaultCcRegistryRegion} gets the default
 * ClearCase registry region, which in turn has properties representing the
 * ClearCase VOBs and views in that region.
 * </li>
 * </bl>
 * </p>
 */
public interface CcProvider 
    extends StpProvider, WorkspaceProvider
{
    /**
     * The name of a CcProvider class whose instances also provide access to a
     * CcProvider object. Pass this name to ProviderFactory.createProvider to
     * obtain an object that implements this interface and provides access to an
     * implementation of CcProvider interface.
     * 
     * @see com.ibm.rational.wvcm.stp.ccex.CcExProvider#NETWORK_PROVIDERS
     * @deprecated Use
     *             com.ibm.rational.wvcm.stp.ccex.CcExProvider#NETWORK_PROVIDERS
     */
    String PROVIDER_CLASS =
        "com.ibm.rational.stp.client.internal.cc.CcSubproviderImpl";

    /**
     * The name of a CcProvider class whose instances provide only support for
     * ClearCase via a server. Pass this name to ProviderFactory.createProvider
     * to obtain an object that implements just this interface.
     */
    String CC_ONLY_PROVIDER_CLASS =
        "com.ibm.rational.stp.client.internal.cc.CcOnlyProviderImpl";

    /**
     * An extension of ProviderFactory.Callback that client call-backs can
     * return to provide a Primary Group or Group List for a ClearCase login
     */
    public interface CcAuthentication extends Authentication {

        /**
         * <p>ClearCase checks the user's "primary group" to determine whether
         * the user can peform certain operations in a given VOB, such as
         * creating a new CC element.
         * </p>
         * <p>On Windows operating systems, the user's primary group cannot be
         * reliably determined, so it must be set explicitly here.
         * </p>
         * <p>On Unix, this setting may be used to override the user's primary
         * group as specified in the /etc/password file or equivalent.
         * </p>
         * @return The primary group name to use performing ClearCase operations,
         *         or <code>null</code> to use the default primary group
         */
        String getPrimaryGroupName();

        /**
         * <p>ClearCase checks the user's group list (the list of OS groups
         * to which the user belongs) to determine whether the user can peform
         * certain operations in a given VOB.
         * </p>
         * <p>If the user belongs to more than 32 groups, in certain situations
         * ClearCase may ignore some of those groups, causing the operation to
         * fail unnecessarily.  In this case, use this setting to define which
         * groups (up to 32) ClearCase should use.
         * </p>
         * @return The group list to use when performing ClearCase operations,
         *         or empty list to use the default group list
         */
        List<String> getGroupList();
    }
    
    /**
     * Get the server's default ClearCase registry region.
     * @param wantedProps list of properties to be returned with registry region proxy
     * @return proxy for server's default registry region.
     * @throws WvcmException
     */
    public CcRegistryRegion doGetDefaultCcRegistryRegion(PropertyRequest wantedProps) throws WvcmException;

    /**
     * <p>
     * Get the default text mode for web views. This depends on two pieces
     * of information: the {@link CcExServer#DEFAULT_VOB_LINE_TERM} property
     * and the client OS. It is defined as follows: 
     * </p>
     * 
     * <table border="1">
     * <tr>
     * <th>server default VOB line term</th>
     * <th>client OS</th>
     * <th>web view default text mode</th>
     * </tr>
     * <td>LF (UNIX)</td>
     * <td>UNIX</td>
     * <td>{@link TextMode#TRANSPARENT}</td>
     * </tr>
     * <tr>
     * <td>LF (UNIX)</td>
     * <td>Windows</td>
     * <td>{@link TextMode#INSERT_CR}</td>
     * </tr>
     * <tr>
     * <td>CR-LF (MSDOS)</td>
     * <td>UNIX</td>
     * <td>{@link TextMode#STRIP_CR}</td>
     * </tr>
     * <tr>
     * <td>CR-LF (MSDOS)</td>
     * <td>Windows</td>
     * <td>{@link TextMode#TRANSPARENT}</td>
     * </tr>
     * <tr>
     * <td>No default set</td>
     * <td>Both UNIX and Windows</td>
     * <td>{@link TextMode#TRANSPARENT}</td>
     * </tr>
     * </table>
     * 
     * @return enumeration representing the view's default text mode
     * @throws WvcmException
     */
    public TextMode doGetDefaultViewTextMode() throws WvcmException;
    
    /**
     * Get the list of web views belonging to the current user, and whose
     * file areas reside on the local machine.
     * @param wantedProps list of properties to retrieve for each web view
     *        proxy.
     * @return list of local web views as a list of CcView proxies
     * @throws WvcmException
     */
    public ResourceList<CcView> getClientViewList(PropertyRequest wantedProps)
        throws WvcmException;

    /**
     * Create a proxy for a ClearCase UCM activity.
     * @param   location  Location for UCM activity.
     * @return  UCM activity proxy.
     */
    public CcActivity ccActivity(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM baseline.
     * @param   location  Location for UCM baseline.
     * @return  UCM baseline proxy.
     */
    public CcBaseline ccBaseline(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM component.
     * @param   location  Location for UCM component.
     * @return  UCM component proxy.
     */
    public CcComponent ccComponent(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM project.
     * @param   location  Location for UCM project.
     * @return  UCM project proxy.
     */
    public CcProject ccProject(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM project folder.
     * @param   location  Location for UCM project folder.
     * @return  UCM project folder proxy.
     */
    public CcProjectFolder ccProjectFolder(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM stream.
     * @param   location  Location for UCM stream.
     * @return  UCM stream proxy.
     */
    public CcStream ccStream(StpLocation location);

    /**
     * Construct a proxy for the CcDirectory (directory in a ClearCase view)
     * specified by the given location.
     * @param loc the location of the directory
     * @return a new CcDirectory proxy for a directory at this Location.
     */
    public CcDirectory ccDirectory(StpLocation loc);

    /**
     * Construct a proxy for the CcFile (file in a ClearCase view)
     * specified by the given location.
     * @param loc the location of the file.
     * @return a new CcFile proxy for a file at this Location.
     */
    public CcFile ccFile(StpLocation loc);

    /**
     * Construct a directory version proxy for the given location.
     * @param loc the location of the directory version.
     * @return a new proxy for a directory version at this Location.
     */
    public CcDirectoryVersion ccDirectoryVersion(StpLocation loc);

    /**
     * Construct a version proxy for the given location.
     * @param loc the location of the version.
     * @return a new proxy for a version at this Location.
     */
    public CcVersion ccVersion(StpLocation loc);

    /**
     * Construct a element proxy for the given location.
     * @param loc the location of the element.
     * @return a new proxy for a element at this Location.
     */
    public CcElement ccElement(StpLocation loc);

    /**
     * Construct a registry region proxy for the given location.
     * @param loc the location of the registry region
     * @return a new proxy for the registry region at this location
     */
    public CcRegistryRegion ccRegistryRegion(StpLocation loc);
    
    /**
     * Construct a VOB proxy for the given location.
     * @param loc the location of the VOB.
     * @return a new proxy for a VOB at this Location.
     */
    public CcVob ccVob(StpLocation loc);

    /**
     * Construct a VOB tag proxy for the given location.
     * @param loc the location of the VOB tag.
     * @return a new proxy for a VOB tag at this Location.
     */
    public CcVobTag ccVobTag(StpLocation loc);

    /**
     * Construct a view proxy for the given location.
     * @param loc the location of the view.
     * @return a new proxy for a view at this Location.
     */
    public CcView ccView(StpLocation loc);

    /**
     * Construct a view tag proxy for the given location.
     * @param loc the location of the view tag.
     * @return a new proxy for a view tag at this Location.
     */
    public CcViewTag ccViewTag(StpLocation loc);

    /**
     * Construct a attribute type proxy for the given location.
     * @param loc the location of the attribute type.
     * @return a new proxy for a attribute type at this Location.
     */
    public CcAttributeType ccAttributeType(StpLocation loc);

    /**
     * Construct a branch type proxy for the given location.
     * @param loc the location of the branch type.
     * @return a new proxy for a branch type at this Location.
     */
    public CcBranchType ccBranchType(StpLocation loc);

    /**
     * Construct a element type proxy for the given location.
     * @param loc the location of the element type.
     * @return a new proxy for a element type at this Location.
     */
    public CcElementType ccElementType(StpLocation loc);

    /**
     * Construct a hyperlink type proxy for the given location.
     * @param loc the location of the hyperlink type.
     * @return a new proxy for a hyperlink type at this Location.
     */
    public CcHyperlinkType ccHyperlinkType(StpLocation loc);

    /**
     * Construct a label type proxy for the given location.
     * @param loc the location of the label type.
     * @return a new proxy for a label type at this Location.
     */
    public CcLabelType ccLabelType(StpLocation loc);

    /**
     * Construct an empty CcLockInfo object for using
     * as a property to set on a CcVobResource.
     * @return a new lock information object
     */
    public CcLockInfo CcLockInfo();
   
    /**
     * Construct a replica proxy for the given location.
     * @param loc the location of the replica.
     * @return a new proxy for a replica at this Location.
     */
    public CcReplica ccReplica(StpLocation loc);

    /**
     * Construct a symbolic link proxy for the given location.
     * @param loc the location of the symbolic link.
     * @return a new proxy for a symbolic link at this Location.
     */
    public CcSymlink ccSymlink(StpLocation loc);

    /**
     * Construct a trigger type proxy for the given location.
     * @param loc the location of the trigger type.
     * @return a new proxy for a trigger type at this Location.
     */
    public CcTriggerType ccTriggerType(StpLocation loc);
    
    /**
     * Register a callback to process ClearPrompt interaction requests.
     * @param callback callback
     */
    public void
    registerClearPromptCallback(CcClearPromptCallback callback);

    /**
     * Register a callback to handle a FileAreaLockedException.
     * @param callback The new callback
     * @return The previous callback
     */
    public CcFileAreaLockedCallback registerCcFileAreaLockedCallback(
            CcFileAreaLockedCallback callback) throws WvcmException;
    
    /**
     * Register a callback to handling manual merges.
     * @param callback The new callback
     * @return The previous callback
     */
    public CcMergeHandlingCallback registerMergeHandlingCallback(
            CcMergeHandlingCallback callback) throws WvcmException;
    
    /**
     * Register a callback to handle state transitions of ClearQuest records. 
     * The callback is invoked if the transition involves required 
     * fields that the user needs to provide.
     * @param callback the callback we're registering
     * @return the previous registered callback
     */
    public CqRecordAutoTransitionCallback registerCqRecordAutoTransitionCallback(
            CqRecordAutoTransitionCallback callback) throws WvcmException;
    
    /**
     * Handle the manual merge using the provider's callback.
     * @return CheckinMergeHandling specifying how the checkin 
     *         is to be handled after the merge
     */
    public CheckinMergeHandling handleManualMerge(
            CcFile file,
            CcVersion fromVersion,
            CcVersion toVersion) throws WvcmException;
}
