/*
 * file CqRecordType.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqRecordType
 *
 *  Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;


import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.io.File;
import java.util.List;

import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.cqex.CqExForm;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;


/**
 * A proxy for the specification of a record type in a database.
 * 
 * The user-friendly specification for the location of a record type has the form
 * <pre>
 *  <b>cq.record:</b><i>&lt;record-type-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 */
public interface CqRecordType
    extends CqUserDbMember, CqUserDb.SearchKey<Boolean>
{
    /** A list of the field definitions of this record type */
    PropertyName<ResourceList<CqFieldDefinition>> FIELD_DEFINITIONS =
        new PropertyName<ResourceList<CqFieldDefinition>>(PROPERTY_NAMESPACE,
                                                          "field-definitions");

    /**
     * Returns the value of the {@link #FIELD_DEFINITIONS FIELD_DEFINITIONS}
     * property as defined by this proxy.
     * 
     * @return A ResourceList of CqFieldDefinition record proxies; one proxy
     *         per field of this record type. Will never by <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #FIELD_DEFINITIONS FIELD_DEFINITIONS} property.
     */
    public ResourceList<CqFieldDefinition> getFieldDefinitions()
        throws WvcmException;

    /** All of the actions defined on this record type. */
    PropertyName<ResourceList<CqAction>> ACTION_LIST =
        new PropertyName<ResourceList<CqAction>>(PROPERTY_NAMESPACE,
                                                  "action-list");

    /**
     * Returns the value of the {@link #ACTION_LIST ACTION_LIST} property as
     * defined by this proxy.
     * 
     * @return A ResourceList of CqAction proxies containing all of the actions
     *         defined for this record type.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ACTION_LIST ACTION_LIST} property.
     */
    public ResourceList<CqAction> getActionList()
        throws WvcmException;

    /**
     * The names of all the states defined for this record type. Will be an
     * empty list if this is a stateless record type.
     */
    PropertyName<List<String>> STATE_NAME_LIST =
        new PropertyName<List<String>>(PROPERTY_NAMESPACE,
                                       "state-name-list");

    /**
     * Returns the value of the {@link #STATE_NAME_LIST STATE_NAME_LIST}
     * property as defined by this proxy.
     *
     * @return  A list of Strings, each containing the name of a state of this
     *          record type. Will be an empty list if this is a stateless 
     *          record type.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #STATE_NAME_LIST STATE_NAME_LIST}
     *                         property.
     */
    public List<String> getStateNameList()
        throws WvcmException;

    /** Answers whether or not this record type is a stateless record type. */
    PropertyName<Boolean> IS_STATELESS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-stateless");

    /**
     * Returns the value of the {@link #IS_STATELESS IS_STATELESS} property as
     * defined by this proxy.
     *
     * @return  <b>true</b> if this is a stateless record type.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #IS_STATELESS IS_STATELESS} property.
     */
    public boolean getIsStateless()
        throws WvcmException;

    /** Answers whether or not records of this type may be submitted by clients. */
    PropertyName<Boolean> IS_SUBMITTABLE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-submittable");

    /**
     * Returns the value of the {@link #IS_SUBMITTABLE IS_SUBMITTABLE} property
     * as defined by this proxy.
     *
     * @return  <b>true</b> if records of this type may be submitted by clients.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #IS_SUBMITTABLE IS_SUBMITTABLE} property.
     */
    public boolean getIsSubmittable()
        throws WvcmException;

    /** All of the named hooks associated with this record type. */
    PropertyName<ResourceList<CqHook>> NAMED_HOOK_LIST =
        new PropertyName<ResourceList<CqHook>>(PROPERTY_NAMESPACE,
                                               "named-hook-list");

    /**
     * Returns the value of the {@link #NAMED_HOOK_LIST NAMED_HOOK_LIST}
     * property as defined by this proxy.
     * 
     * @return A ResourceList of CqHook proxies containing all the named hooks
     *         of this record type.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #NAMED_HOOK_LIST NAMED_HOOK_LIST} property.
     */
    public ResourceList<CqHook> getNamedHookList()
        throws WvcmException;

    /** Answers whether or not records of this type can be security contexts. */
    PropertyName<Boolean> CAN_BE_SECURITY_CONTEXT =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "can-be-security-context");

    /**
     * Returns the value of the {@link #CAN_BE_SECURITY_CONTEXT
     * CAN_BE_SECURITY_CONTEXT} property as defined by this proxy.
     * 
     * @return <b>true</b> if records of this type can be security contexts
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CAN_BE_SECURITY_CONTEXT CAN_BE_SECURITY_CONTEXT}
     *             property.
     */
    public boolean getCanBeSecurityContext()
        throws WvcmException;

    /**
     * Answers whether or a record of this record type is used as a security
     * context
     */
    PropertyName<Boolean> IS_SECURITY_CONTEXT =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-security-context");

    /**
     * Returns the value of the {@link #IS_SECURITY_CONTEXT IS_SECURITY_CONTEXT}
     * property as defined by this proxy.
     *
     * @return  <b>true</b> if records of this type are used as security
     *          contexts.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #IS_SECURITY_CONTEXT IS_SECURITY_CONTEXT}
     *                         property.
     */
    public boolean getIsSecurityContext()
        throws WvcmException;

    /** 
     * Answers whether or not this is the definition of a record type family. 
     */
    PropertyName<Boolean> IS_FAMILY =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-family");

    /**
     * Returns the value of the {@link #IS_FAMILY IS_FAMILY} property as defined
     * by this proxy.
     * 
     * @return <b>true</b> if this is a record type family specification.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_FAMILY IS_FAMILY} property.
     */
    public boolean getIsFamily()
        throws WvcmException;
    
    /**
     * Answers whether or not this record type is a built-in record type, not
     * added by a schema developer or package.
     */
    PropertyName<Boolean> IS_BUILT_IN_SYSTEM_OWNED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-built-in-system-owned");

    /**
     * Returns the value of the
     * {@link #IS_BUILT_IN_SYSTEM_OWNED IS_BUILD_IN_SYSTEM_OWNED} property as
     * defined by this proxy.
     * 
     * @return <b>true</b> if this record type is defined for all ClearQuest
     *         schemas as part of the base implementation.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_BUILT_IN_SYSTEM_OWNED IS_BUILT_IN_SYSTEM_OWNED}
     *             property.
     */
    public boolean getIsBuiltInSystemOwned()
        throws WvcmException;
    
    /**
     * The record types that are members of this record type family. The list
     * is empty is this is not a record type family.
     */
    PropertyName<ResourceList<CqRecordType>> FAMILY_MEMBERS =
        new PropertyName<ResourceList<CqRecordType>>(PROPERTY_NAMESPACE,
                                                     "family-members");
    
    /**
     * Returns the value of the {@link #FAMILY_MEMBERS} property as defined by
     * this proxy
     * 
     * @return A ResourceList containing a CqRecordType proxy for each record
     *         type that is a member of this record type family; Will never be
     *         <b>null</b> but may be empty if this is not a record type family
     *         or if no members are defined.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #FAMILY_MEMBERS} property.
     */
    public ResourceList<CqRecordType> getFamilyMembers() 
        throws WvcmException;
    
    /**
     * A list of the record type families this record is a member of.
     */
    PropertyName<ResourceList<CqRecordType>> PARENT_FAMILIES =
        new PropertyName<ResourceList<CqRecordType>>(PROPERTY_NAMESPACE, 
                                                     "parent-families");
    
    /**
     * Returns the value of the {@link #PARENT_FAMILIES} property as defined by
     * this proxy
     * 
     * @return A ResourceList containing a CqRecordType proxy for each record
     *         type family that this record type is a member of; Will never be
     *         <b>null</b> but may be empty if this record type is not a member
     *         of a record type family.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #PARENT_FAMILIES} property.
     */
    public ResourceList<CqRecordType> getParentFamilies() 
        throws WvcmException;
        
    /**
     * A list of the schema-defined fields that constitute the unique key for
     * records of this type, presented in sort-key order starting from the
     * primary sort key. (Note: In this release computation of this property
     * requires the IS_RAW_SQL_WRITER permission.)
     */
    PropertyName<ResourceList<CqFieldDefinition>> KEY_FIELDS =
        new PropertyName<ResourceList<CqFieldDefinition>>(PROPERTY_NAMESPACE,
                                                              "key-fields");
    
    /**
     * Returns the value of the {@link CqRecordType#KEY_FIELDS KEY_FIELDS} property as
     * defined by this proxy.
     *
     * @return  A ResourceList of CqFieldDefinition proxies each representing a
     *          key field of this record type presented in sort-key order..
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link CqRecordType#KEY_FIELDS KEY_FIELDS} property.
     */
    public ResourceList<CqFieldDefinition> getKeyFields() throws WvcmException;

    /**
     * Answers whether or not records of this type may be used as a UCM
     * activity.
     */
    PropertyName<Boolean> IS_UCM_ENABLED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-ucm-enabled");

    /**
     * Returns the value of the {@link #IS_UCM_ENABLED IS_UCM_ENABLED} property
     * as defined by this proxy.
     * 
     * @return <b>true</b> if records of this type may be used as a UCM
     *         activity.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_UCM_ENABLED IS_UCM_ENABLED} property.
     */
    public boolean getIsUcmEnabled()
        throws WvcmException;
    
    /**
     * Returns specified fields from records that are of this record type and
     * match a specified filtering expression. This proxy does not need to
     * define any properties as only its location is needed to establish the
     * record type to be selected and the database from which to select them.
     * 
     * @param displayFields An array of {@link CqQuery.DisplayField} objects,
     *            each specifying a value to be returned from each selected
     *            record and collectively specifying the order in which the
     *            matched records are to be included in the response. Must not
     *            be <b>null</b>.
     * @param filtering A {@link CqQuery.Filter} object specifying how the
     *            records of this type are to be selected for inclusion in the
     *            response. May be <b>null</b>, in which case all records of
     *            this record type will be included (up to the limits imposed by
     *            the targetRow and maxRows parameters).
     * @param targetRow Specifies the first record that is to be included in the
     *            response among the records that match the filter, where a
     *            value of 1 specifies the first matching record. The ordering
     *            of matched records is defined by the display fields array.
     * @param maxRows The maximum number of records that should be included in
     *            the response to this request. A value of Long.MAX_VALUE will
     *            request that the targeted record and all records after it in
     *            the prescribed ordering should be included in the response; If
     *            this parameter is 0, no records will be included. If the
     *            filter matches fewer than the maximum records requested, it is
     *            not an error.
     * @param options An instance of {@link CqQuery.ListOptions} that specifies
     *            optional attributes for controlling the performance of the
     *            query or its response. May be <b>null</b>, in which case
     *            server-specified values will be used for each attribute.
     * 
     * @return A CqResultSet of CqRowData objects each representing a record
     *         selected by the filter and containing the values specified by the
     *         display fields array. The values in each CqRowData object are
     *         ordered according to the display fields array. The total row
     *         count, if requested, is available as a field of the returned
     *         CqResultSet.
     * 
     * @throws WvcmException if any errors occur during the execution of the
     *             query or the formation of the response.
     *             
     * @see CqQuery#doExecute(long, long, com.ibm.rational.wvcm.stp.cq.CqQuery.ListOptions, com.ibm.rational.wvcm.stp.cq.CqQuery.FilterLeaf[])
     */
    CqResultSet doQuery(CqQuery.DisplayField[] displayFields,
                        CqQuery.Filter filtering,
                        long targetRow,
                        long maxRows,
                        CqQuery.ListOptions options)
        throws WvcmException;

    /**
     * Returns specified fields from records that are of the type defined by
     * this record type and match a specified filtering expression. This proxy
     * does not need to define any properties as only its location is needed to
     * establish the type of records to be selected and the database from which
     * to select them.
     * 
     * @param displayFields An array of {@link CqQuery.DisplayField} objects,
     *            each specifying a value to be returned from each selected
     *            record and collectively specifying the order in which the
     *            matched records are to be included in the response. Must not
     *            be <b>null</b>.
     * @param filtering A {@link CqQuery.Filter} node specifying how the records
     *            of this type are to be selected for inclusion in the response.
     *            May be <b>null</b>, in which case all records of this record
     *            type will be included (up to the limits imposed by the
     *            targetRow and maxRows parameters).
     * @param options An instance of {@link CqQuery.FileOptions} that specifies
     *            optional attributes for controlling the format of the response
     *            response. May be <b>null</b>, in which case server-specified
     *            values will be used for each attribute.
     * @param feedback A Feedback instance specifying properties to be returned
     *            from the operation in the CqRecordType proxy. May be <b>null</b>.
     * 
     * @param result The File into which the result set is to be written. Must
     *            not be <b>null</b>.
     * 
     * @return A proxy for this record type, populated with the properties
     *         requested by the Feedback argument.
     * 
     * @throws WvcmException if any errors occur during the execution of the
     *             query or the formation of the response.
     */
    CqRecordType doQuery(CqQuery.DisplayField[] displayFields,
                         CqQuery.Filter filtering,
                         File result,
                         Feedback feedback,
                         CqQuery.FileOptions options)
        throws WvcmException;

    /**
     * Returns specified fields from records that are of the type defined by
     * this record type and match a specified filtering expression. To launch
     * this method this proxy does not need to define any properties as only its
     * location is needed to establish the type of records to be selected and
     * the database from which to select them.
     * 
     * @param sql A String containing an SQL select statement specifying the
     *            fields to be returned by the query and the filter to be used
     *            to select the records.
     * 
     * @param targetRow Specifies the first record that is to be included in the
     *            response among the records that match the filter, where a
     *            value of 1 specifies the first matching record. The ordering
     *            of matched records is defined by the display fields array.
     * 
     * @param maxRows The maximum number of records that should be included in
     *            the response to this request. A value of Long.MAX_VALUE will
     *            request that the targeted record and all records after it in
     *            the prescribed ordering should be included in the response; If
     *            this parameter is 0, no records will be included. If the
     *            filter matches fewer than the maximum records requested, it is
     *            not an error.
     * 
     * @param options An instance of {@link CqQuery.ListOptions} that specifies
     *            optional attributes for controlling the performance of the
     *            query or its response. May be <b>null</b>, in which case
     *            server-specified values will be used for each attribute.
     * 
     * @return A CqResultSet containing a list of CqRowData objects, each
     *         representing a record selected by the filter and containing the
     *         values specified by the SQL string. The values in each CqRowData
     *         object are ordered according to the SQL select statement.
     * 
     * @throws WvcmException if any errors occur during the execution of the
     *             query or the formation of the response.
     */
    CqResultSet doQuery(String sql, 
                        long targetRow, 
                        long maxRows,
                        CqQuery.ListOptions options)
    throws WvcmException;

    /**
     * Returns specified fields from records that are of the type defined by
     * this record type and match a specified filtering expression. To launch
     * this method this proxy does not need to define any properties as only its
     * location is needed to establish the type of records to be selected and
     * the database from which to select them.
     * 
     * @param sql A String containing an SQL specification for the fields to be
     *            returned by the query and the filter to be used to select the
     *            records.
     * 
     * @param result The File into which the result set is to be written. Must
     *            not be <b>null</b>.
     * 
     * @param feedback A Feedback instance specifying properties to be returned
     *            from the operation in the CqRecordType proxy. May be <b>null</b>.
     * 
     * @param options An instance of FileOptions that specifies optional
     *            attributes for controlling the format of the response
     *            response. May be <b>null</b>, in which case server-specified
     *            values will be used for each attribute.
     * 
     * @return A proxy for this record type, populated with the properties
     *         requested by the Feedback argument.
     * 
     * @throws WvcmException if any errors occur during the execution of the
     *             query or the formation of the response.
     */
    CqRecordType doQuery(String sql, 
                         File result,
                         Feedback feedback,
                         CqQuery.FileOptions options)
    throws WvcmException;
}
